@@ -18,6 +18,100 @@ from txrdq.rdq import ResizableDispatchQueue, QueueStopped
 from txrdq.job import Job
 
 
+class TestPendingStops(unittest.TestCase):
+    """
+    Tests that check that the number of pending stops gets set correctly
+    following changes in queue width.
+    """
+
+    # These tests all return dq.stop() because if the queue is set to be
+    # very wide, the reactor will be unclean when the test ends as many
+    # Deferreds will be being created.
+
+    def testZeroOnDefaultCreation(self):
+        """
+        A new queue whose width is unspecified should have no pending
+        stops.
+        """
+        dq = ResizableDispatchQueue(None)
+        self.assertEqual(0, dq.pendingStops)
+        return dq.stop()
+
+    def testZeroOnExplicitCreation(self):
+        """
+        A new queue whose width is 10 should have no pending stops.
+        """
+        dq = ResizableDispatchQueue(None, 10)
+        self.assertEqual(0, dq.pendingStops)
+        return dq.stop()
+
+    def testNarrow(self):
+        """
+        Create a queue of width 5 and narrow it to width 3. There should
+        then be 2 pending stops.
+        """
+        dq = ResizableDispatchQueue(None, 5)
+        dq.width = 3
+        self.assertEqual(2, dq.pendingStops)
+        return dq.stop()
+
+    def testNarrowWiden(self):
+        """
+        Create a queue of width 10. Narrowing to 7 means there will be 3
+        pending stops, then widening to 8 will decrease the number of
+        pending stops to 2.
+        """
+        dq = ResizableDispatchQueue(None, 10)
+        dq.width = 7
+        self.assertEqual(dq.pendingStops, 3)
+        dq.width = 8
+        self.assertEqual(dq.pendingStops, 2)
+        return dq.stop()
+
+    def testSetImmediatelyToZero(self):
+        """
+        Create a queue of width 10 and then set its width to 0.
+        The number of pending stops will be 10.
+        """
+        dq = ResizableDispatchQueue(None, 10)
+        dq.width = 0
+        self.assertEqual(dq.pendingStops, 10)
+        return dq.stop()
+
+    def testIncrementToVeryHighValue(self):
+        """
+        Create a queue of width 10 and then set its width to 1000.
+        The number of pending stops will be 0.
+        """
+        dq = ResizableDispatchQueue(None, 10)
+        dq.width = 1000
+        self.assertEqual(dq.pendingStops, 0)
+        return dq.stop()
+
+    def testIncrementToVeryHighValueThenSetToLowValue(self):
+        """
+        Create a queue of width 10 and then set its width to 1000 then set
+        it to 100.  The number of pending stops will be 900.
+        """
+        dq = ResizableDispatchQueue(None, 10)
+        dq.width = 1000
+        dq.width = 100
+        self.assertEqual(dq.pendingStops, 900)
+        return dq.stop()
+
+    def testIncrementToVeryHighValueThenSetToLowValueThenIncrement(self):
+        """
+        Create a queue of width 10 and then set its width to 1000 then set
+        it to 100, then to 500.  The number of pending stops will be 500.
+        """
+        dq = ResizableDispatchQueue(None, 10)
+        dq.width = 1000
+        dq.width = 100
+        dq.width = 500
+        self.assertEqual(dq.pendingStops, 500)
+        return dq.stop()
+
+
 class TestSimple(unittest.TestCase):
     """
     Simple tests of the ResizableDispatchQueue class.
@@ -56,7 +150,23 @@ class TestSimple(unittest.TestCase):
         dq.width = 0
         dq.put('aaa')
         dq.put('bbb')
+        # import time
+        # time.sleep(0.10)
         self.assertEqual((0, 2), dq.size())
+        return dq.stop()
+
+    @defer.inlineCallbacks
+    def testSetWidthToZeroAfterInitiallyNonZeroThenStop(self):
+        """
+        Make sure that a queue whose width is initially non-zero and which
+        is then set to zero width returns all the added jobs when stopped.
+        """
+        dq = ResizableDispatchQueue(None, 3)
+        dq.width = 0
+        dq.put('aaa', 5)
+        dq.put('bbb', 10)
+        remaining = yield dq.stop()
+        self.assertEqual(['aaa', 'bbb'], [job.jobarg for job in remaining])
 
     @defer.inlineCallbacks
     def testStopSetsStopped(self):
@@ -130,21 +240,6 @@ class TestSimple(unittest.TestCase):
         dq.setWidth(3)
         self.assertEqual(dq.width, 3)
 
-    def testNarrowWiden(self):
-        """
-        A tests that exercises the code to reduce the number of pending
-        stops in the queue. Narrowing to 7 means there will be 3 pending
-        stops, then widening to 8 will decrease the number of pending stops
-        by one.
-        """
-        dq = ResizableDispatchQueue(None, 10)
-        dq.setWidth(7)
-        self.assertEqual(dq.pendingStops, 3)
-        self.assertEqual(dq.width, 7)
-        dq.setWidth(8)
-        self.assertEqual(dq.pendingStops, 2)
-        self.assertEqual(dq.width, 8)
-
     @defer.inlineCallbacks
     def testPauseResumeWithZeroWidth(self):
         """
@@ -513,3 +608,18 @@ class TestTiming(unittest.TestCase):
         yield task.deferLater(reactor, 0.1, dq.pause)
         pendingJobs = dq.pending()
         self.assertEqual(pendingJobs, [])
+
+    @defer.inlineCallbacks
+    def testSetWidthToZeroAfterInitiallyNonZero(self):
+        """
+        Make sure that a queue whose width is initially non-zero and which
+        is then set to zero width does not then begin to process any added
+        jobs.
+        """
+        dq = ResizableDispatchQueue(self.slow, 3)
+        yield task.deferLater(reactor, 0.01, dq.setWidth, 0)
+        reactor.callLater(0.01, dq.put, 'aaa')
+        reactor.callLater(0.01, dq.put, 'bbb')
+        yield task.deferLater(reactor, 0.05, self._testSize, dq, (0, 2))
+        remaining = yield dq.stop()
+        self.assertEqual(['aaa', 'bbb'], [job.jobarg for job in remaining])
